"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DaikinOnePlusPlatform = void 0;
const settings_1 = require("./settings");
const platformThermostat_1 = require("./platformThermostat");
const platformAQI_1 = require("./platformAQI");
const platformHumidity_1 = require("./platformHumidity");
const platformAwaySwitch_1 = require("./platformAwaySwitch");
const daikinapi_1 = require("./daikinapi");
/**
 * HomebridgePlatform
 * This class is the main constructor for your plugin, this is where you should
 * parse the user config and discover/register accessories with Homebridge.
 */
class DaikinOnePlusPlatform {
    constructor(log, config, api) {
        this.log = log;
        this.config = config;
        this.api = api;
        this.Service = this.api.hap.Service;
        this.Characteristic = this.api.hap.Characteristic;
        // this is used to track restored cached accessories
        this.accessories = [];
        //Don't start if not configured
        if (!this.config) {
            this.log.error('Not configured.');
            return;
        }
        if (!this.config.user || !this.config.password) {
            this.log.error('No Daikin login credentials configured.');
        }
        if (!this.config.refreshInterval) {
            this.config.refreshInterval = 10;
            this.log.warn('Refresh Interval not set. Using default of 10 seconds.');
        }
        this.log.debug(`Using refresh interval of ${this.config.refreshInterval} seconds`);
        if (this.config.includeDeviceName === undefined) {
            this.config.includeDeviceName = false;
            this.log.warn('Include Device Name not set. Using default of false.');
        }
        this.log.debug(`Using Include Device Name setting of ${this.config.includeDeviceName}`);
        this.log.debug(`Finished initializing platform: ${this.config.name}`);
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const logMessage = (level, message, ...parameters) => {
            let logLevel = "info" /* INFO */;
            switch (level) {
                case "info" /* INFO */:
                    logLevel = "info" /* INFO */;
                    break;
                case "warn" /* WARN */:
                    logLevel = "warn" /* WARN */;
                    break;
                case "error" /* ERROR */:
                    logLevel = "error" /* ERROR */;
                    break;
                case "debug" /* DEBUG */:
                    logLevel = "debug" /* DEBUG */;
                    break;
            }
            this.log.log(logLevel, message, parameters);
        };
        this.daikinApi = new daikinapi_1.DaikinApi(this.config.user, this.config.password, this.config.refreshInterval, logMessage);
        // When this event is fired it means Homebridge has restored all cached accessories from disk.
        // Dynamic Platform plugins should only register new accessories after this event was fired,
        // in order to ensure they weren't added to homebridge already. This event can also be used
        // to start discovery of new accessories.
        //Set first delay to -refreshInterval to immediately refresh
        this.api.on("didFinishLaunching" /* DID_FINISH_LAUNCHING */, this.discover.bind(this, this.config.refreshInterval * -1));
    }
    discover(delay = 0) {
        this.log.debug('Executed didFinishLaunching callback');
        clearTimeout(this.discoverTimer);
        //If initialized, no need to try and discover devices again.
        if (this.daikinApi.isInitialized()) {
            return;
        }
        const refresh = this.config.refreshInterval + delay;
        this.discoverTimer = setTimeout(() => {
            void (async () => {
                // run the method to discover / register your devices as accessories
                await this.discoverDevices();
                //Call discover again in case we failed to initialize the api and discover devices.
                this.discover();
            })();
        }, refresh * 1000);
    }
    /**
     * This function is invoked when homebridge restores cached accessories from disk at startup.
     * It should be used to setup event handlers for characteristics and update respective values.
     */
    configureAccessory(accessory) {
        this.log.info('Loading accessory from cache:', accessory.displayName);
        // add the restored accessory to the accessories cache so we can track if it has already been registered
        this.accessories.push(accessory);
    }
    /**
     * This is an example method showing how to register discovered accessories.
     * Accessories must only be registered once, previously created accessories
     * must not be registered again to prevent "duplicate UUID" errors.
     */
    async discoverDevices() {
        if (!this.config.user && !this.config.password) {
            this.log.error('Credentials not set. Aborting discovery of devices.');
            return;
        }
        await this.daikinApi.Initialize();
        if (!this.daikinApi.isInitialized()) {
            this.log.error('Unable to retrieve devices. Aborting discovery of devices.');
            return;
        }
        const devices = await this.daikinApi.getDeviceList();
        // loop over the discovered devices and register each one if it has not already been registered
        for (const device of devices) {
            this.log.info(`Found device: ${device.name}`);
            const deviceData = await this.daikinApi.getDeviceData(device.id);
            this.discoverThermostat(device);
            this.discoverOutdoorHumSensor(device);
            this.discoverIndoorHumSensor(device);
            this.discoverOutdoorAqi(device, deviceData);
            this.discoverIndoorAqi(device, deviceData);
            this.discoverAwaySwitch(device);
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    discoverAwaySwitch(device) {
        const uuid = this.api.hap.uuid.generate(`${device.id}_away`);
        this.log.info('Checking for Away Switch...');
        const existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
        if (this.config.enableAwaySwitch) {
            const dName = this.config.includeDeviceName ? `${device.name} Away State` : 'Away State';
            if (existingAccessory) {
                // the accessory already exists
                existingAccessory.displayName = dName;
                this.log.info('Restoring existing away switch from cache:', existingAccessory.displayName);
                new platformAwaySwitch_1.DaikinOnePlusAwaySwitch(this, existingAccessory, device.id, this.daikinApi, dName);
            }
            else {
                // the accessory does not yet exist, so we need to create it
                this.log.info('Adding new away switch:', dName);
                const accessory = new this.api.platformAccessory(dName, uuid);
                accessory.context.device = device;
                new platformAwaySwitch_1.DaikinOnePlusAwaySwitch(this, accessory, device.id, this.daikinApi, dName);
                this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
            }
        }
        else if (existingAccessory) {
            //Delete any existing Away switch
            this.log.info('Removing Away switch from cache:', existingAccessory.displayName);
            this.api.unregisterPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [existingAccessory]);
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    discoverIndoorAqi(device, deviceData) {
        const uuid = this.api.hap.uuid.generate(`${device.id}_iaqi`);
        this.log.info('Checking for indoor Air Quality sensor...');
        const existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
        if (!this.config.ignoreIndoorAqi) {
            const dName = this.config.includeDeviceName ? `${device.name} Indoor AQI` : 'Indoor AQI';
            if (deviceData.aqIndoorAvailable) {
                if (existingAccessory) {
                    // the accessory already exists
                    existingAccessory.displayName = dName;
                    this.log.info('Restoring existing indoor Air Quality sensor from cache:', existingAccessory.displayName);
                    new platformAQI_1.DaikinOnePlusAQSensor(this, existingAccessory, device.id, this.daikinApi, true, dName);
                }
                else {
                    // the accessory does not yet exist, so we need to create it
                    this.log.info('Adding new indoor Air Quality sensor:', dName);
                    const accessory = new this.api.platformAccessory(dName, uuid);
                    accessory.context.device = device;
                    new platformAQI_1.DaikinOnePlusAQSensor(this, accessory, device.id, this.daikinApi, true, dName);
                    this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                }
            }
            else if (existingAccessory) {
                this.log.info('Removing legacy indoor Air Quality Sensor from cache:', existingAccessory.displayName);
                this.api.unregisterPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [existingAccessory]);
            }
        }
        else if (existingAccessory) {
            //Delete any existing Indoor AQI switch
            this.log.info('Removing Indoor AQI sensor from cache:', existingAccessory.displayName);
            this.api.unregisterPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [existingAccessory]);
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    discoverOutdoorAqi(device, deviceData) {
        const uuid = this.api.hap.uuid.generate(`${device.id}_oaqi`);
        this.log.info('Checking for outdoor Air Quality sensor...');
        const existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
        if (!this.config.ignoreOutdoorAqi) {
            const dName = this.config.includeDeviceName ? `${device.name} Outdoor AQI` : 'Outdoor AQI';
            if (deviceData.aqOutdoorAvailable) {
                if (existingAccessory) {
                    // the accessory already exists
                    existingAccessory.displayName = dName;
                    this.log.info('Restoring existing outdoor Air Quality sensor from cache:', existingAccessory.displayName);
                    new platformAQI_1.DaikinOnePlusAQSensor(this, existingAccessory, device.id, this.daikinApi, false, dName);
                }
                else {
                    // the accessory does not yet exist, so we need to create it
                    this.log.info('Adding new outdoor Air Quality sensor:', dName);
                    const accessory = new this.api.platformAccessory(dName, uuid);
                    accessory.context.device = device;
                    new platformAQI_1.DaikinOnePlusAQSensor(this, accessory, device.id, this.daikinApi, false, dName);
                    this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                }
            }
            else if (existingAccessory) {
                this.log.info('Removing legacy outdoor Air Quality Sensor from cache:', existingAccessory.displayName);
                this.api.unregisterPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [existingAccessory]);
            }
        }
        else if (existingAccessory) {
            //Delete any existing Outdoor AQI sensor
            this.log.info('Removing Outdoor AQI sensor from cache:', existingAccessory.displayName);
            this.api.unregisterPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [existingAccessory]);
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    discoverIndoorHumSensor(device) {
        const uuid = this.api.hap.uuid.generate(`${device.id}_ihum`);
        this.log.info('Checking for indoor humidity sensor...');
        const existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
        if (!this.config.ignoreIndoorHumSensor) {
            const dName = this.config.includeDeviceName ? `${device.name} Indoor Humidity` : 'Indoor Humidity';
            if (existingAccessory) {
                // the accessory already exists
                existingAccessory.displayName = dName;
                this.log.info('Restoring existing indoor humidity sensor from cache:', existingAccessory.displayName);
                new platformHumidity_1.DaikinOnePlusHumidity(this, existingAccessory, device.id, this.daikinApi, true);
            }
            else {
                // the accessory does not yet exist, so we need to create it
                this.log.info('Adding new indoor humidity sensor:', dName);
                const accessory = new this.api.platformAccessory(dName, uuid);
                accessory.context.device = device;
                new platformHumidity_1.DaikinOnePlusHumidity(this, accessory, device.id, this.daikinApi, true);
                this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
            }
        }
        else if (existingAccessory) {
            //Delete any existing Indoor Humidity sensor
            this.log.info('Removing Indoor Humidity sensor from cache:', existingAccessory.displayName);
            this.api.unregisterPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [existingAccessory]);
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    discoverOutdoorHumSensor(device) {
        const uuid = this.api.hap.uuid.generate(`${device.id}_ohum`);
        this.log.info('Checking for outdoor humidity sensor...');
        const existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
        if (!this.config.ignoreOutdoorHumSensor) {
            const dName = this.config.includeDeviceName ? `${device.name} Outdoor Humidity` : 'Outdoor Humidity';
            if (existingAccessory) {
                // the accessory already exists
                existingAccessory.displayName = dName;
                this.log.info('Restoring existing outdoor humidity sensor from cache:', existingAccessory.displayName);
                new platformHumidity_1.DaikinOnePlusHumidity(this, existingAccessory, device.id, this.daikinApi, false);
            }
            else {
                // the accessory does not yet exist, so we need to create it
                this.log.info('Adding new outdoor humidity sensor:', dName);
                const accessory = new this.api.platformAccessory(dName, uuid);
                accessory.context.device = device;
                new platformHumidity_1.DaikinOnePlusHumidity(this, accessory, device.id, this.daikinApi, false);
                this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
            }
        }
        else if (existingAccessory) {
            //Delete any existing Outdoor Humidity sensor
            this.log.info('Removing Outdoor Humidity sensor from cache:', existingAccessory.displayName);
            this.api.unregisterPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [existingAccessory]);
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    discoverThermostat(device) {
        const uuid = this.api.hap.uuid.generate(`${device.id}_tstat`);
        this.log.info('Checking for thermostat...');
        const existingAccessory = this.accessories.find(accessory => accessory.UUID === uuid);
        if (!this.config.ignoreThermostat) {
            const dName = this.config.includeDeviceName ? `${device.name} Thermostat` : 'Thermostat';
            if (existingAccessory) {
                // the accessory already exists
                existingAccessory.displayName = dName;
                this.log.info('Restoring existing thermostat from cache:', existingAccessory.displayName);
                new platformThermostat_1.DaikinOnePlusThermostat(this, existingAccessory, device.id, this.daikinApi);
            }
            else {
                // the accessory does not yet exist, so we need to create it
                this.log.info('Adding new thermostat:', dName);
                const accessory = new this.api.platformAccessory(dName, uuid);
                accessory.context.device = device;
                new platformThermostat_1.DaikinOnePlusThermostat(this, accessory, device.id, this.daikinApi);
                this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
            }
        }
        else if (existingAccessory) {
            this.log.info('Removing Thermostat from cache:', existingAccessory.displayName);
            this.api.unregisterPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [existingAccessory]);
        }
    }
}
exports.DaikinOnePlusPlatform = DaikinOnePlusPlatform;
//# sourceMappingURL=platform.js.map